import { PlatformEventEmitter, OpenAi } from '@modal-sh/mio-ai';

export const addEvents = (aiClient: PlatformEventEmitter, resolve?: Function, reject?: Function) => {
  let result: Partial<OpenAi.ChatCompletion> | undefined;

  aiClient.on<OpenAi.ChatCompletionChunkDataEvent>('data', (d) => {
    d.choices.forEach((c) => {
      if (!result) {
        result = {
          id: d.id,
          object: OpenAi.ChatCompletionDataEventObjectType.CHAT_COMPLETION,
          created: d.created,
          model: d.model,
        };
      }

      if (!Array.isArray(result?.choices)) {
        result.choices = [];
      }

      if (!result.choices[c.index]) {
        result.choices[c.index] = {
          message: { content: '' },
          index: c.index,
          finish_reason: c.finish_reason,
        };
      }

      if (result.choices[c.index].message) {
        if (c.delta.role) {
          (result.choices[c.index].message as Record<string, unknown>).role = c.delta.role;
        }

        if (c.delta.content) {
          (result.choices[c.index].message as Record<string, unknown>)
            .content += c.delta.content;
        }
      }

      if (c.finish_reason) {
        result.choices[c.index].finish_reason = c.finish_reason;
      }
    });
  });

  aiClient.on('end', () => {
    resolve?.(result);
  });

  aiClient.on('error', (error: Error) => {
    reject?.(error);
  });
};
